/*
   $Id: BlackList.h 648 2011-10-31 12:59:22Z manu $
*/
/*
 * Copyright INRA/CNRS - Thomas.Faraut@toulouse.inra.fr, Emmanuel.Courcelle@toulouse.inra.fr
 * This software is a computer program whose purpose is to align nucleic sequences, or perform other utility tasks
 * This software is governed by the CeCILL-C license under French law and abiding by the rules of distribution of free software.
 * 
 * Please read the file Licence_CeCILL-C_V1-en.txt which should have been distributed with this file
 * The fact that you are presently reading this, and the aforementioned file, means that you have had
 * knowledge of the CeCILL-C license and that you accept its terms.
*/


#ifndef _BLACKLIST_H_
#define _BLACKLIST_H_

#include <set>
#include <vector>
#include "ptypes.h"
#include "window.h"

/*
    The class BlackList handles the 'black-listed' words (words deleted because they were too frequent)
    This is a sery simplistic hash table, built upon a vector and a lot of set objects
    You can only insert some words to the table, and ask the table to know if some given word is
    in the table or not. That's all we need yet
*/

class BlackList {
    public:

    /* The constructors/destructor - We specifye the size of the hash at start */
    BlackList(int);

    /* Inserting a new word in the list */
    void Insert(const Word&);

    /* Searching a word - return true if the word is in the list, false if not */
    bool LookUp(const Word&) const;

    /* Printing the list */
    friend ostream & operator<<(ostream& os, const BlackList &);

    private:
    vector< set<Word> > hash_table;
    int length;     // The number of sets
};

/*
     The inline member functions of the class: BlackList
*/

inline BlackList::BlackList(int d): length(d)
{
    set<Word> s;
    for (int i=0;i<length;i++)
        hash_table.push_back(s);
}

inline void BlackList::Insert(const Word &w)
{
    int code=w.Hash(length);
    set<Word>&s = hash_table[code];
    s.insert(w);
}

inline bool BlackList::LookUp(const Word & w) const
{
    int code=w.Hash(length);
    const set<Word>& s = hash_table[code];
    return (s.find(w)!=s.end());
}

#endif

